/* $Workfile: OtxEthD.h $   $Revision: 5 $    $Modtime: 5/15/08 1:38p $  */
#ifndef OTX_ETH_DEF_H
#define OTX_ETH_DEF_H
/*******************************************************************************

  Copyright (C) 2008 Odin TeleSystems Inc., All Rights Reserved

  @doc PublicHwApi OtxEthD

  OTX Generic Hardware Driver API

  @module
  OtxEthD | Constants, Macros, and Types for the Ethernet Switch Device

  @contents2
  OtxEthD.h |

  @index const enum struct type | OtxEthD

  @contents2
  OTX Ethernet Switch Device Constants, Macros, and Types | Types and macros for the Ethernet Switch device

  @head3 Enumerated Types  |

  @index enum | OtxEthD

  @head3 Struct Types |

  @index struct | OtxEthD

  @head3 Constants |

  @index const | OtxEthD

  @head3 Other Types |

  @index type | OtxEthD

*******************************************************************************/

#include "OtxType.h"

#ifdef __cplusplus
  extern "C" {
#endif

#if defined(__linux__)
#pragma pack(4)
#elif defined(__BORLANDC__)
#pragma option push -a4
#else
#pragma pack(push,4)
#endif


// @const Number of DMP ports (10/100) that the Ethernet switch can handle. The port numbers for these ports range from 0 through 7.
#define OTX_ETH_RMAC_PORTS 8 

// @const Port number of the CPU port for the Ethernet switch
#define OTX_ETH_CMAC_PORT 8 

// @const Port number of the GBE (Gigabit Ethernet) port for the Ethernet switch. This port is also referred to tas the WAN port.
#define OTX_ETH_GMAC_PORT 9 

// @const Number of bytes in a control command and a control response. See <f OtxEthControlCommand>.
#define OTX_ETH_CTRL_CMD_FRAME_SIZE 40



/*******************************************************************************
  @enum 
    Attributes for the Ethernet switch
*******************************************************************************/
enum OtxEthAttributesE {
  OTX_ATTR_ETH_REPORT_MAC_AND_VLAN          = OTX_ATTR_END + 0, // @emem (OTX_ATTRIB_BOOL) Read/Write. Default is OTX_TRUE.
                                                                // <nl> This attributes specifies whether the Ethernet Switch device will report 
                                                                // VLAN port association, learned and deleted (aging) MAC addresses events. 
                                                                // See <e OtxEthEventCodesE.OTX_ETH_EC_LEARNED_MAC_ADDRESS> and
                                                                // <e OtxEthEventCodesE.OTX_ETH_EC_DELETED_MAC_ADDRESS>.

  OTX_ATTR_ETH_REPORT_WRAPPED_COUNTERS      = OTX_ATTR_END + 1, // @emem (OTX_ATTRIB_BOOL) Read/Write. Default is OTX_TRUE.
                                                                // <nl> This attributes specifies whether the Ethernet Switch device will report 
                                                                // wrapped counter events.
                                                                // See <e OtxEthEventCodesE.OTX_ETH_EC_COUNTER_WRAPPED_TOTAL_FRAMES_RECEIVED> and
                                                                // similar "COUNTER_WRAPPED" events.
  
  
  OTX_ATTR_ETH_END                          = OTX_ATTR_END + 2
};

typedef enum OtxEthAttributesE OtxEthAttributesE;




/*******************************************************************************
  @enum
    Result Codes

*******************************************************************************/
enum OtxEthResultCodesE {
  OTX_ETH_RES_UNKNOWN_MODE             = OTX_RES_END + 0  , // @emem The device is not setup in a valid mode.                 
  OTX_ETH_RES_INVALID_CHANNEL          = OTX_RES_END + 1  , // @emem Invalid channel.                                         
  OTX_ETH_RES_UNSUPPORTED_VER          = OTX_RES_END + 2  , // @emem Unsupported Device version.                              
  OTX_ETH_RES_END                      = OTX_RES_END + 3
};

typedef enum OtxEthResultCodesE OtxEthResultCodesE;


/*******************************************************************************
  @enum
    Ethernet Switch Event Codes

*******************************************************************************/
enum OtxEthEventCodesE {
  OTX_ETH_EC_COUNTER_WRAPPED_BYTES_SENT                     = 0x00, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nBytesSent>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_UNICAST_FRAME_SENT             = 0x01, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nUnicastFramesSent>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_FRAME_SENT_FAIL                = 0x02, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nFrameTransFailures>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_FLOW_CONTROL_SENT              = 0x03, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nFlowCtrlFramesSent>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_NON_UNICAST_FRAME_SENT         = 0x04, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nNonUnicastFrmsSent>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_BYTES_RECEIVED                 = 0x05, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nBytesReceived>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_FRAME_RECEIVED                 = 0x06, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nFramesReceived>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_TOTAL_BYTES_RECEIVED           = 0x07, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nTotalBytesReceived>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_TOTAL_FRAMES_RECEIVED          = 0x08, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nTotalFramesReceived>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_FLOW_CONTROL_FRAMES_RECEIVED   = 0x09, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nFlowCtrlFramesReceived>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_MULTICAST_FRAMES_RECEIVED      = 0x0A, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nMulticastFramesReceived>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_BROADCAST_FRAMES_RECEIVED      = 0x0B, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nBroadcastFramesReceived>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_FRAMES_LEN_64_BYTES            = 0x0C, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nFramesWithLen64>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_JABBER_FRAMES                  = 0x0D, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nJabberFrames>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_FRAMES_LEN_65_TO_127_BYTES     = 0x0E, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nFramesLen65To127>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_OVERSIZE_FRAMES                = 0x0F, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nOversizeFrames>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_FRAMES_LEN_128_TO_255_BYTES    = 0x10, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nFramesLen128To255>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_FRAMES_LEN_256_TO_512_BYTES    = 0x11, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nFramesLen256To511>. <e OtxEventDataS.m_nParam> of the event contains the port number. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_FRAMES_LEN_512_TO_1023_BYTES   = 0x12, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nFramesLen512To1023>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_FRAMES_LEN_1024_TO_1528_BYTES  = 0x13, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nFramesLen1024ToMax>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_FRAGMENT                       = 0x14, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nFragments>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_ALIGNMENT_ERROR                = 0x15, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nAlignmentErrors>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_UNDERSIZED_FRAMES              = 0x16, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nUndersizeFrames>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_CRC_ERRORS                     = 0x17, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nCrcErrors>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_SHORT_EVENT                    = 0x18, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nShortEvents>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_COLLISION                      = 0x19, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nCollisions>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_DROP                           = 0x1A, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nFramesDroppedCongestion>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_FRAMES_FILTERED                = 0x1B, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nFramesFilteredByEngine>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_COUNTER_WRAPPED_LATE_COLLISION                 = 0x1C, // @emem Statistical counter has wrapped. See <e OtxEthStatCountersS.nLateCollision>. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_LINK_DOWN                                      = 0x1D, // @emem Link status change. Ethernet link is down. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_LINK_UP                                        = 0x1E, // @emem Link status change. Ethernet link is up. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_MODULE_DETECTED                                = 0x1F, // @emem Module Detect status change. Module detected. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_NO_MODULE                                      = 0x20, // @emem Module Detect status change. No module. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_LHB_FRAME_DETECTED                             = 0x21, // @emem LHB Status change. LHB frame detected. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_LHB_RECEIVING_TIMEOUT                          = 0x22, // @emem LHB Status change. LHB receving timeout. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_QOS_STATUS                                     = 0x23, // @emem QoS status. 
                                                                    // <nl> <e OtxEventDataS.m_nParam> of the event contains the port number. 
                                                                    // <nl> <e OtxEventDataS.m_nRequestId> of the event contains the QoS status (8 bits for RMAC ports, and 16 bits for the GMAC port).
  OTX_ETH_EC_PREAMPLE_INFO_DETECTED                         = 0x24, // @emem GBE port. Preample info detected. <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_NO_PREAMPLE                                    = 0x25, // @emem GBE port. No Preample  <e OtxEventDataS.m_nParam> of the event contains the port number.
  OTX_ETH_EC_LEARNED_MAC_ADDRESS                            = 0x26, // @emem New MAC address learned by the search engine. 
                                                                    // <nl> Bit [11:0] of the <e OtxEventDataS.m_nParam> of the event contains the 12-bit VLANID. 
                                                                    // <nl> Bit [14:12] of the <e OtxEventDataS.m_nParam> of the event contains the 3 bit status (described below):
                                                                    // <nl> Status bits:
                                                                    // <nl> 000 - Invalid entry
                                                                    // <nl> 001 - Dynamic MAC Address
                                                                    // <nl> 010 - Reserved
                                                                    // <nl> 011 - Static MAC Address
                                                                    // <nl> 100 - Reserved
                                                                    // <nl> 101 - Static MAC Address with source and destination filter
                                                                    // <nl> 110 - Static MAC Address with source filter
                                                                    // <nl> 111 - Static MAC Address with destination filter
                                                                    // <nl> The MAC address is located in the following bytes of the event:
                                                                    // <nl> First  (from the right) MAC address byte: Bit [23:16] of <e OtxEventDataS.m_nParam>
                                                                    // <nl> Second (from the right) MAC address byte: Bit [31:24] of <e OtxEventDataS.m_nParam>
                                                                    // <nl> Third  (from the right) MAC address byte: Bit [7:0]   of <e OtxEventDataS.m_nRequestId>
                                                                    // <nl> Forth  (from the right) MAC address byte: Bit [15:8]  of <e OtxEventDataS.m_nRequestId>
                                                                    // <nl> Fifth  (from the right) MAC address byte: Bit [23:16] of <e OtxEventDataS.m_nRequestId>
                                                                    // <nl> Sixth  (from the right) MAC address byte: Bit [31:24] of <e OtxEventDataS.m_nRequestId>
  OTX_ETH_EC_DELETED_MAC_ADDRESS                            = 0x27, // @emem Deleted aged MAC address from the search engine. The 12-bit VLANID is passed in the event parameter
                                                                    // <nl> Bit [11:0] of the <e OtxEventDataS.m_nParam> of the event contains the 12-bit VLANID. 
                                                                    // <nl> The MAC address is located in the following bytes of the event:
                                                                    // <nl> First  (from the right) MAC address byte: Bit [23:16] of <e OtxEventDataS.m_nParam>
                                                                    // <nl> Second (from the right) MAC address byte: Bit [31:24] of <e OtxEventDataS.m_nParam>
                                                                    // <nl> Third  (from the right) MAC address byte: Bit [7:0]   of <e OtxEventDataS.m_nRequestId>
                                                                    // <nl> Forth  (from the right) MAC address byte: Bit [15:8]  of <e OtxEventDataS.m_nRequestId>
                                                                    // <nl> Fifth  (from the right) MAC address byte: Bit [23:16] of <e OtxEventDataS.m_nRequestId>
                                                                    // <nl> Sixth  (from the right) MAC address byte: Bit [31:24] of <e OtxEventDataS.m_nRequestId>

  OTX_ETH_EC_END_OF_EVENT_CODES   // @emem Internal use

};

typedef enum OtxEthEventCodesE OtxEthEventCodesE;




/*******************************************************************************

  @enum
    Link Status

  @xref
    <f OtxEthGetLinkStatus>

*******************************************************************************/
enum OtxEthLinkStatusE {
  OTX_ETH_LINK_DOWN = 0,  // @emem The Ethernet link is down (not connected?)
  OTX_ETH_LINK_UP   = 1   // @emem The Ethernet link is up
};

typedef enum OtxEthLinkStatusE OtxEthLinkStatusE;


/*******************************************************************************

  @struct
    Port status and configuration of an Ethernet Switch device port .

  @xref
   <f OtxEthGetPortStatus>

*******************************************************************************/
struct OtxEthPortStatusS {
  OTX_BOOL     bLinkUp;                   // @field If OTX_TRUE then the link on this port is up (operational). If OTX_FALSE then the link on this port is down (disconnected; not operational). 
  OTX_BOOL     bFlowControl;              // @field If OTX_TRUE then this port is configured to use Flow control. If OTX_FALSE then this port is not configured to use Flow Control.
  OTX_BOOL     bFullDuplex;               // @field If OTX_TRUE then this port is configured to use Full Duplex. If OTX_FALSE then this port is configured to use Half Duplex.
  OTX_BOOL     nLinkSpeed;                // @field Linkspeed in Megabits (10, 100, or 1000) if the link is up (<e OtxEthPortStatusS.bLinkUp is OTX_TRUE>).
};

typedef struct OtxEthPortStatusS OtxEthPortStatusS;



/*******************************************************************************

  @struct
    Statistics counters for the Ethernet Switch device (per port).

  @xref
   <f OtxEthGetStatCounters>

*******************************************************************************/
struct OtxEthStatCountersS {
  OTX_UINT32   nBytesReceived;             // @field (32 bits) Reserved for future use. Counter is always set to zero at this time.
  OTX_UINT32   nFramesReceived;            // @field (32 bits) Number of frames received
  OTX_UINT32   nTotalBytesReceived;        // @field (32 bits) Reserved for future use. Counter is always set to zero at this time.
  OTX_UINT32   nTotalFramesReceived;       // @field (32 bits) Number of frames received (good)
  OTX_UINT32   nFlowCtrlFramesReceived;    // @field (8 bits)  Number of flow control frames received
  OTX_UINT32   nMulticastFramesReceived;   // @field (16 bits) Number of multicast frames received
  OTX_UINT32   nBroadcastFramesReceived;   // @field (16 bits) Number of broadcast frames received
  OTX_UINT32   nBytesSent;                 // @field (32 bits) Reserved for future use. Counter is always set to zero at this time.
  OTX_UINT32   nUnicastFramesSent;         // @field (24 bits) Number of unicast frames sent
  OTX_UINT32   nNonUnicastFrmsSent;        // @field (16 bits) Number of non-unicast frames sent
  OTX_UINT32   nFlowCtrlFramesSent;        // @field (16 bits) Number of flow control frames sent
  OTX_UINT32   nFramesWithLen64;           // @field (24 bits) Number of frames with length of 64 bytes
  OTX_UINT32   nFramesLen65To127;          // @field (24 bits) Number of frames with length in the range of 65 bytes to 127 bytes
  OTX_UINT32   nFramesLen128To255;         // @field (16 bits) Number of frames with length in the range of 128 bytes to 255 bytes
  OTX_UINT32   nFramesLen256To511;         // @field (16 bits) Number of frames with length in the range of 256 bytes to 511 bytes
  OTX_UINT32   nFramesLen512To1023;        // @field (16 bits) Number of frames with length in the range of 512 bytes to 1023 bytes
  OTX_UINT32   nFramesLen1024ToMax;        // @field (16 bits) Number of frames with length in the range of 1024 bytes to maximum frame size bytes
  OTX_UINT32   nOversizeFrames;            // @field (8 bits)  Number of oversize frames (frames with a length greater than maximum frame size or frames with a length greater than 1522 but less then or equal to BUF_LIMIT (default 0x40) with with CRC error)
  OTX_UINT32   nJabberFrames;              // @field (8 bits)  Number of jabber frames (frames > 6400 bytes in size)
  OTX_UINT32   nFragments;                 // @field (16 bits) Number of fragments
  OTX_UINT32   nShortEvents;               // @field (16 bits) Number of short events
  OTX_UINT32   nFrameTransFailures;        // @field (8 bits)  Number of frame transmission failures
  OTX_UINT32   nAlignmentErrors;           // @field (8 bits)  Number of alignment errors (also increments the <e OtxEthStatCountersS.nCrcErrors> counter)
  OTX_UINT32   nUndersizeFrames;           // @field (8 bits)  Number of undersize frames (frames of size less than 64 bytes)
  OTX_UINT32   nCrcErrors;                 // @field (16 bits) Number of CRC errors (note: frames of length greater than 1522 bytes but less than or equal to BUF_LIMIT (default 0x40)  with CRC errors will be counted as <e OtxEthStatCountersS.nOversizeFrames> frames)
  OTX_UINT32   nCollisions;                // @field (16 bits) Number of collisions
  OTX_UINT32   nFramesDroppedCongestion;   // @field (16 bits) Number of frames dropped due to TX (egress) queue congestion
  OTX_UINT32   nFramesFilteredByEngine;    // @field (16 bits) Number of frames filtered by search engine
  OTX_UINT32   nLateCollision;             // @field (8 bits)  Number of late collisions

};

typedef struct OtxEthStatCountersS OtxEthStatCountersS;








#if defined(__linux__)
#pragma pack()
#elif defined(__BORLANDC__)
#pragma option pop
#else
#pragma pack(pop)
#endif

#ifdef __cplusplus
  } // extern "C"
#endif


/*******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 2008


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

*******************************************************************************/
#endif // OTX_ETH_DEF_H
